const entities = {}
const isExist = (entityType, name) => {
  return (entities.hasOwnProperty(entityType) && entities[entityType].hasOwnProperty(name))
}

class EntityFabric {
  /**
   * Constructor
   *
   * @param types [array]
   */
  constructor ({types = []}) {
    this.types = []
    if (types instanceof Array) {
      this.types = types
    }
    this.types.forEach((type) => {
      entities[type] = {}
    })
  }

  /**
   * Add new entity
   *
   * @param entityType [string]
   * @param name [string]
   * @param data [any]
   * @returns {boolean}
   */
  add (entityType, name, data) {
    if (this.types.indexOf(entityType) !== -1) {
      entities[entityType][name] = {name, entity: data}
      return true
    }
    return false
  }

  /**
   * Get Entity data
   *
   * @param entityType
   * @param name
   * @returns {*}
   */
  get (entityType, name) {
    if (isExist(entityType, name)) {
      return entities[entityType][name]
    }
    return false
  }

  /**
   * Remove Entity data
   *
   * @param entityType
   * @param name
   * @returns {boolean}
   */
  remove (entityType, name) {
    if (isExist(entityType, name)) {
      delete entities[entityType][name]
      return true
    }
    return false
  }
  raw () {
    return {...entities}
  }
}

/**
 * get EntityFabric instance
 *
 * @param obj
 * @returns {EntityFabric}
 */
module.exports = (obj = {}) => new EntityFabric(obj)
